
// stuff

	#include <windows.h>
	#include <string.h>
	#include <stdio.h>
	
	int string_is_here (char* a, char* b) {
		while (1) {
			if (!*b) return 1; // b ran out; b is entirely contained in a
			if (!*a) return 0; // a ran out, didn't find all of b in it
			if (*a != *b) return 0; // different character found, no match
			a ++;
			b ++;
		}
	}
	int strings_are_equal (char* a, char* b) {
		while (1) {
			if (!*b && !*a) return 1; // both ran out, match!
			if (*a != *b) return 0; // different character found, no match
			a ++;
			b ++;
		}
	}

// things

	static inline void move_window_to (HWND window, int x, int y) {
		RECT rect = {0};
		GetWindowRect(window, &rect);
		MoveWindow(window, x, y, rect.right-rect.left, rect.bottom-rect.top, FALSE);
	}
	static inline void resize_window_height (HWND window, int h) {
		RECT rect = {0};
		GetWindowRect(window, &rect);
		MoveWindow(window, rect.left, rect.top, rect.right, h, FALSE);
		
		// NOTE/TODO: when the window resizes, the expanded area shows up as pure white. I can't figure out how to make it refresh programmatically.
		// Some ideas: If the white area scrolls off-screen, it will become cleared, so maybe you can scroll the window down and back up somehow to redraw it. If you resize a console manually with your mouse, it won't become white, so maybe there's another way to resize the window than MoveWindow().
		// UpdateWindow(window);
		// RedrawWindow(window, NULL, NULL, RDW_ERASE|RDW_FRAME|RDW_INTERNALPAINT|RDW_ERASENOW|RDW_UPDATENOW);
		// SendMessage(window, WM_PAINT, 0, 0);
	}
	static inline RECT get_window_rect (HWND window) {
		RECT rect = {0};
		GetWindowRect(window, &rect);
		return rect;
	}
	static inline int get_screen_width (void) {
		return GetSystemMetrics(SM_CXSCREEN);
	}
	static inline int get_screen_height (void) {
		return GetSystemMetrics(SM_CYSCREEN);
	}
	
	int main (int argc, char** args) {
		if (argc <= 1) {
			printf("--- SETCMD ---\n"
			       " .. Move window:\n"
			       "       top=X    -> Move window to X pixels from the top of the monitor.\n"
			       "       bottom=X -> Move window to X pixels from the bottom of the monitor.\n"
			       "       left=X   -> Move window to X pixels from the left side of the monitor.\n"
			       "       right=X  -> Move window to X pixels from the right side of the monitor.\n"
			       " .. Change window size:\n"
			       "       height=X -> Set window height to X pixels (cmd.exe defaults to ~300).\n"
			       " .. Note: =X can be omitted, in that case height defaults to screen height, and positions default to 0.\n"
			       " .. Example: 'setcmd top left height=800'\n"
			       "--- SETCMD ---\n");
			return 0;
		}
		
		HWND window = GetConsoleWindow();
		
		// for (int i=0; i<argc; i++) {
		// 	printf("ARG (%s)\n", args[i]);
		// }
		
		int bottom = -1;
		int top = -1;
		int left = -1;
		int right = -1;
		int height = -1;
		
		int screenh = get_screen_height();
		int screenw = get_screen_width();
		
		for (int i=1; i<argc; i++) {
			char* arg = args[i];
			int len = strlen(arg);
			
			if (string_is_here(arg, "top")) {
				// printf("MOVE %s\n", arg);
				top = 0;
				bottom = -1;
				arg += strlen("top");
				if (*arg == '=') {
					arg ++;
					int val = strtol(arg,NULL,0);
					if (val) top = val;
				}
			}
			else if (string_is_here(arg, "bottom")) {
				// printf("MOVE %s\n", arg);
				top = -1;
				bottom = 0;
				arg += strlen("bottom");
				if (*arg == '=') {
					arg ++;
					int val = strtol(arg,NULL,0);
					if (val) bottom = val;
				}
			}
			else if (string_is_here(arg, "left")) {
				// printf("MOVE %s\n", arg);
				left = 0;
				right = -1;
				arg += strlen("left");
				if (*arg == '=') {
					arg ++;
					int val = strtol(arg,NULL,0);
					if (val) left = val;
				}
			}
			else if (string_is_here(arg, "right")) {
				// printf("MOVE %s\n", arg);
				left = -1;
				right = 0;
				arg += strlen("right");
				if (*arg == '=') {
					arg ++;
					int val = strtol(arg,NULL,0);
					if (val) right = val;
				}
			}
			else if (string_is_here(arg, "height")) {
				// printf("HEIGHT %s ", args[i]);
				int amount = 0;
				
				arg += strlen("height");
				if (*arg == '=') {
					arg ++;
					int val = strtol(arg,NULL,0);
					amount = val;
				}
				else {
					amount = screenh;
				}
				
				if (amount > screenh) amount = screenh;
				
				if (amount > 50) {
					resize_window_height(window, amount);
				}
			}
			else {
				printf("cmdset: Unknown argument %s\n", arg);
			}
		}
		
		RECT rect = get_window_rect(window);
		int cmdh = rect.bottom - rect.top;
		int cmdw = rect.right - rect.left;
		if (bottom >= 0) {
			top = screenh - cmdh - bottom;
			if (top < 0) top = 0;
		}
		if (right >= 0) {
			left = screenw - cmdw - right;
			if (left < 0) left = 0;
		}
		
		if (left >= 0 || top >= 0) {
			if (left < 0) left = rect.left;
			if (top < 0) top = rect.top;
			move_window_to(window, left, top);
		}
		
		// getchar();
		
		return 0;
	}
